==> GeneticsBase_1.0.0.tar.gz: GeneticsBase/R/diseq.ci.R <==
diseq.ci <- function(object, marker, R=1000,
                       conf=0.95, correct=TRUE,
                       na.rm=TRUE, ...)
  {
    
    ## step 1 - generate summary table
    levs <- unlist(alleleLevels(object,marker))
    flag <- levs %in% missingCodes(object, marker, drop=FALSE)
    levs <- levs[!flag]
    
    observed.no <-
      table(
            factor(
                   alleles(object, which=1, marker=marker),
                   levels = levs
                   )
            , 
            factor(
                   alleles(object, which=2, marker=marker),
                   levels = levs
                   )
            )
    
    observed <- prop.table(observed.no)
    observed <- 1/2 * (observed + t(observed) )
    
    ## step 2 - make table into a probability vector for calling rmultinom
    n <- sum(observed.no)
    prob.vector <- c(observed)
    
    ## step 3 - sample R multinomials with the specified frequenceis
    ## (include observed data to avoid bias)
    resample.data <- cbind(c(observed.no),
                           rmultz2( n, prob.vector, R ) )
    
    
    bootfun <- function(x) {
      observed[,] <- x/n
      observed <- 1/2 * (observed + t(observed) )
      d <-  diseq.table(observed)
      c("Overall D "=d$D.overall,
        "Overall D'"=d$Dprime.overall,
        "Overall r "=d$r.overall)
    }
    
    results <- apply( resample.data, 2, bootfun )
    
    alpha.2 <- (1-conf)/2
    
    ##  ci <- t(apply(results, 1,
    ##              quantile, c( alpha.2 , 1-alpha.2), na.rm=TRUE ))

    if(length(levs)<=2)
      {
        ci <- t(apply(results, 1, function(x) quantile(x, c(0.025, 0.975),
                                                     na.rm=na.rm ) ) )
        warning.text <- NULL
      }
    else
      {
        warning.text <- paste("For more than two alleles, overall",
                              "disequlibrium statistics are bounded",
                              "between [0,1].  Because of this, confidence",
                              "intervals for values near 0 and 1 are",
                              "ill-behaved.", sep=" ")
        
        if(correct)
          {
            warning.text <- paste(warning.text, "A rough correction has been applied, but",
                                  "the intervals still may not be correct for values near 0 or 1.",
                                  sep=" ")
            
          ci <- t(apply(results, 1,
                        function(x)
                        ci.balance(x,x[1],confidence=conf,
                                   minval=0,maxval=1)$ci ))
          }
        else
          ci <- t(apply(results, 1, function(x) quantile(x, c(0.025, 0.975) ) ) )
        
        warning(paste(strwrap(c(warning.text,"\n"),prefix="  "),collapse="\n") )
      }

    
    na.count <-  function(x) sum(is.na(x))
    nas <- apply( results, 1, na.count)
    
    zero.in.range <- (ci[,1] <= 0) & (ci[,2] >= 0)
    
    ci <- cbind( "Observed"=results[,1], ci, "NAs"=nas,
                "Zero in Range"=zero.in.range )
    
    outside.ci <- (ci[,1] < ci[,2]) | (ci[,1] > ci[,3])
    
    if( any(outside.ci) )
      warning("One or more observed value outide of confidence interval. Check results.")
    
    if(any(nas>0))
      warning("NAs returned from diseq call")
    
    
    retval <- list(
                   call=match.call(),
                   R=R,
                   conf=conf,
                   ci=ci,
                   warning.text=warning.text
                   )
    retval
  }

==> GeneticsBase_1.14.0.tar.gz: GeneticsBase/R/diseq.ci.R <==
diseq.ci <- function(object, marker, R=1000,
                       conf=0.95, correct=TRUE,
                       na.rm=TRUE, ...)
  {
    
    ## step 1 - generate summary table
    levs <- unlist(alleleLevels(object,marker))
    flag <- levs %in% missingCodes(object, marker, drop=FALSE)
    levs <- levs[!flag]
    
    observed.no <-
      table(
            factor(
                   alleles(object, which=1, marker=marker),
                   levels = levs
                   )
            , 
            factor(
                   alleles(object, which=2, marker=marker),
                   levels = levs
                   )
            )
    
    observed <- prop.table(observed.no)
    observed <- 1/2 * (observed + t(observed) )
    
    ## step 2 - make table into a probability vector for calling rmultinom
    n <- sum(observed.no)
    prob.vector <- c(observed)
    
    ## step 3 - sample R multinomials with the specified frequenceis
    ## (include observed data to avoid bias)
    resample.data <- cbind(c(observed.no),
                           rmultz2( n, prob.vector, R ) )
    
    
    bootfun <- function(x) {
      observed[,] <- x/n
      observed <- 1/2 * (observed + t(observed) )
      d <-  diseq.table(observed)
      c("Overall D "=d$D.overall,
        "Overall D'"=d$Dprime.overall,
        "Overall r "=d$r.overall)
    }
    
    results <- apply( resample.data, 2, bootfun )
    
    alpha.2 <- (1-conf)/2
    
    ##  ci <- t(apply(results, 1,
    ##              quantile, c( alpha.2 , 1-alpha.2), na.rm=TRUE ))

    if(length(levs)<=2)
      {
        ci <- t(apply(results, 1, function(x) quantile(x,
                                                       c(0.025, 0.975),
                                                       na.rm=na.rm
                                                       )
                      ) )
        warning.text <- NULL
      }
    else
      {
        warning.text <- paste("For more than two alleles, overall",
                              "disequlibrium statistics are bounded",
                              "between [0,1].  Because of this, confidence",
                              "intervals for values near 0 and 1 are",
                              "ill-behaved.", sep=" ")
        
        if(correct)
          {
            warning.text <- paste(warning.text, "A rough correction has been applied, but",
                                  "the intervals still may not be correct for values near 0 or 1.",
                                  sep=" ")
            
          ci <- t(apply(results, 1,
                        function(x)
                        ci.balance(x,x[1],confidence=conf,
                                   minval=0,maxval=1)$ci ))
          }
        else
          ci <- t(apply(results, 1, function(x) quantile(x, c(0.025, 0.975), na.rm=na.rm ) ) )
        
        warning(paste(strwrap(c(warning.text,"\n"),prefix="  "),collapse="\n") )
      }

    
    na.count <-  function(x) sum(is.na(x))
    nas <- apply( results, 1, na.count)
    
    zero.in.range <- (ci[,1] <= 0) & (ci[,2] >= 0)
    
    ci <- cbind( "Observed"=results[,1], ci, "NAs"=nas,
                "Zero in Range"=zero.in.range )
    
    outside.ci <- (ci[,1] < ci[,2]) | (ci[,1] > ci[,3])
    
    if( any(outside.ci) )
      warning("One or more observed value outide of confidence interval. Check results.")
    
    if(any(nas>0))
      warning("NAs returned from diseq call")
    
    
    retval <- list(
                   call=match.call(),
                   R=R,
                   conf=conf,
                   ci=ci,
                   warning.text=warning.text
                   )
    retval
  }
